import json
from logging import Logger

from cvss.cvss3 import CVSS3
from rule_engine import Rule, SymbolResolutionError, RuleSyntaxError
from .manualVettingException import ManualVettingException


class CvssLib:

    @classmethod
    def __init__(cls, rules_file_path: str):
        """
        :param rules_file_path: str
        :rtype: object
        """
        cls.rules_actions = cls.__get_rules_actions(
            rules_file_path=rules_file_path)

    @classmethod
    def get_modified_cvss(cls,
                          record: dict,
                          original_vector_string: str,
                          logger: Logger) -> tuple:
        """
        :param record: dict - This is a single vulnerability record from your json output file
        :param original_vector_string: str
        :param logger: Logger - passed in from the calling script
        :return:
        :rtype: tuple - modified_vector_string, modified_environmental_score, \
            modified_severity, rules_applied
        :exception: SymbolResolutionError will log to error if the path in your
            custom rule cannot be found in the source json file
        :exception: RuleSyntaxError will log to error if the rule you have
            defined cannot be parsed.
        :exception: ManualVettingException will be thrown if no rules were matched.
            This can be caught in your parent script
        """
        original_vector_obj = cls.__str2dict(original_vector_string)
        if original_vector_obj['CVSS'].startswith('3'):
            cvss_obj = CVSS3(original_vector_string)
        elif original_vector_obj['CVSS'].startswith('2'):
            raise ValueError(f'CVSS version {original_vector_obj["CVSS"]} is unsupported, manual vetting required')
        results = []
        rules_applied = []
        result = False
        for rule_action in cls.rules_actions:
            try:
                rule = Rule(rule_action['rule'])
                result = rule.matches(record)
            except SymbolResolutionError as srerr:
                logger.error(f'{srerr.message}. The {srerr.symbol_name} block in the rules_actions.json file '
                             f'was not found in the json record')
            # if true, we need to update the vector string and return it.
            except RuleSyntaxError as rserr:
                logger.error(f'{rserr.message}. The block value in the '
                             f'rules_actions.json file '
                             f'contains incorrect syntax.')
            if result is True:
                rules_applied.append({
                    "description": rule_action['description'],
                    "vector_changes": rule_action["vector_changes"]
                })
                for vector_change in rule_action["vector_changes"]:
                    cvss_obj.metrics[vector_change['vector']] = vector_change['value']
                cvss_obj.compute_temporal_score()
                cvss_obj.compute_environmental_score()
            results.append(result)

        if True not in results:
            raise ManualVettingException("No rescore rules were matched by this record. Manual vetting is required!")

        modified_vector_string = f'CVSS:3.{cvss_obj.minor_version}/{cls.__dict2str(cvss_obj.metrics)}'
        modified_environmental_score = cvss_obj.environmental_score
        modified_severity = cvss_obj.severities()
        return modified_vector_string, modified_environmental_score, \
            modified_severity, rules_applied

    @classmethod
    def __str2dict(cls, vector_string: str) -> dict:
        vector_dict = {}
        vector_parts = vector_string.split('/')
        for vector_part in vector_parts:
            key = vector_part.split(':')[0]
            value = vector_part.split(':')[1]
            vector_dict[key] = value

        return vector_dict

    @classmethod
    def __dict2str(cls, vector_object: dict) -> str:
        vector_string = ''
        for key in vector_object.keys():
            value = vector_object[key]
            vector_part = f'{key}:{value}'
            vector_string = f'{vector_string}/{vector_part}'

        # remove leading /
        vector_string = vector_string.lstrip('/')
        return vector_string

    @classmethod
    def __get_rules_actions(cls, rules_file_path: str) -> dict:
        with open(rules_file_path, 'r') as r:
            rules_actions = json.load(r)
        return rules_actions



import logging
import os
from pathlib import Path

import pytest
from rule_engine import RuleSyntaxError, SymbolResolutionError

from cvss_rescore.cvsslib import CvssLib
from cvss_rescore.cvsslib import ManualVettingException

logger = logging.getLogger(__name__)


rules_file_path = Path(__file__).parent.parent
rules_file = os.path.join(rules_file_path, 'rules_actions.json')


def test_cvss2_throws_value_error():
    record = {
        "locations": "",
        "package": {
            "Package": {
                "pm": "npm",
                "group": None,
                "name": "minimist",
                "version": "1.2.0",
                "vendor": None,
                "fixVersions": ["[1.2.6]"],
                "impactPaths": [["npm://covert:1.0.0", "npm://minimist:1.2.0"]]
            },
            "Vulnerabilities": [{
                "id": "XRAY-000000",
                "title": "Critical vulnerability found in component temp_react_core",
                "description": "Prototype pollution vulnerability in function parseQuery in parseQuery.js in webpack loader-utils 2.0.0 via the name variable in parseQuery.js.",
                "cvssScore": "10.0",
                "cvssVector": "CVSS:2/AV:N/AC:L/Au:N/C:C/I:C/A:C)",
                "cve": "CVE-2022-00000"
            }]
        }
    }
    original_vector_string = record.get('package').get('Vulnerabilities')[0].get('cvssVector')
    cvsslib = CvssLib(rules_file_path=rules_file)
    try:
        modified_vector_string, modified_environmental_score, modified_severity, rules_applied = cvsslib.get_modified_cvss(record=record, original_vector_string=original_vector_string, logger=logger)
    except ValueError as ve:
        assert True


def test_cvss3_does_not_throw_exception():
    record = {
        "locations": "",
        "package": {
            "Package": {
                "pm": "npm",
                "group": None,
                "name": "minimist",
                "version": "1.2.0",
                "vendor": None,
                "fixVersions": ["[1.2.6]"],
                "impactPaths": [["npm://covert:1.0.0", "npm://minimist:1.2.0"]]
            },
            "Vulnerabilities": [{
                "id": "XRAY-000000",
                "title": "Critical vulnerability found in component temp_react_core",
                "description": "Prototype pollution vulnerability in function parseQuery in parseQuery.js in webpack loader-utils 2.0.0 via the name variable in parseQuery.js.",
                "cvssScore": "10.0",
                "cvssVector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                "cve": "CVE-2022-00000"
            }]
        }
    }
    original_vector_string = record.get('package').get('Vulnerabilities')[0].get('cvssVector')
    cvsslib = CvssLib(rules_file_path=rules_file)
    modified_vector_string, \
        modified_environmental_score, \
        modified_severity, \
        rules_applied = cvsslib.get_modified_cvss(
        record=record,
        original_vector_string=original_vector_string,
        logger=logger)
    assert modified_severity[0] == 'Critical'


def test_invalid_vector_value():
    record = {
        "locations": "",
        "package": {
            "Package": {
                "pm": "npm",
                "group": None,
                "name": "Invalid_Vector_Value_Test",
                "version": "1.0.0",
                "vendor": None,
                "fixVersions": ["[1.2.6]"],
                "impactPaths": [["npm://covert:1.0.0", "npm://Invalid_Vector_Value_Test:1.0.0"]]
            },
            "Vulnerabilities": [{
                "id": "XRAY-000000",
                "title": "Critical vulnerability found in component temp_react_core",
                "description": "Prototype pollution vulnerability in function parseQuery in parseQuery.js in webpack loader-utils 2.0.0 via the name variable in parseQuery.js.",
                "cvssScore": "9.8",
                "cvssVector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                "cve": "CVE-2022-00000"
            }]
        }
    }
    original_vector_string = record.get('package').get('Vulnerabilities')[0].get('cvssVector')
    cvsslib = CvssLib(rules_file_path=rules_file)
    try:
        modified_vector_string, modified_environmental_score, modified_severity, rules_applied = cvsslib.get_modified_cvss(record=record, original_vector_string=original_vector_string, logger=logger)
    except RuleSyntaxError as rse:
        print(rse.message)
        assert True


def test_invalid_rule_attribute_not_found():
    record = {
        "locations": "",
        "package": {
            "Package": {
                "pm": "npm",
                "group": None,
                "name": "Invalid_Vector_Value_Test",
                "version": "1.0.0",
                "vendor": None,
                "fixVersions": ["[1.2.6]"],
                "impactPaths": [["npm://covert:1.0.0", "npm://Invalid_Vector_Value_Test:1.0.0"]]
            },
            "Vulnerabilities": [{
                "id": "XRAY-000000",
                "title": "Critical vulnerability found in component temp_react_core",
                "description": "Prototype pollution vulnerability in function parseQuery in parseQuery.js in webpack loader-utils 2.0.0 via the name variable in parseQuery.js.",
                "cvssScore": "9.8",
                "cvssVector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                "cve": "CVE-2022-00000"
            }]
        }
    }
    original_vector_string = record.get('package').get('Vulnerabilities')[0].get('cvssVector')
    cvsslib = CvssLib(rules_file_path=rules_file)
    try:
        modified_vector_string, modified_environmental_score, modified_severity, rules_applied = cvsslib.get_modified_cvss(record=record, original_vector_string=original_vector_string, logger=logger)
    except SymbolResolutionError as sre:
        print(sre.message)
        assert True



import logging
import os
import sys

import requests as requests
import semver
from dotenv import load_dotenv
from requests import Response

FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
formatter = logging.Formatter(FORMAT)
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler(stream=sys.stdout)
ch.setFormatter(formatter)
logger.addHandler(ch)


def get_github_latest_release(token: str, owner: str, repo: str) -> Response:
    logger.info(f'{owner}/{repo}')
    headers = {
        'Accept': 'application/vnd.github+json',
        'Authorization': f'Bearer {token}',
        'X-GitHub-Api-Version': '2022-11-28',
    }

    response = requests.get(
        url=f'https://api.github.com/repos/{owner}/{repo}/releases/latest',
        headers=headers)
    logger.info(response.json())
    return response


def create_github_release(
        token: str,
        owner: str,
        repo: str,
        tag: str,
        generate_release_notes: bool):
    generate_release_notes = str(generate_release_notes).lower()
    headers = {
        'Accept': 'application/vnd.github+json',
        'Authorization': f'Bearer {token}',
        'X-GitHub-Api-Version': '2022-11-28',
    }

    data = '{' \
           f'"tag_name":"{tag}",' \
           '"target_commitish":"main",' \
           f'"name":"{tag}",' \
           '"body":"",' \
           '"draft":false,' \
           '"prerelease":false,' \
           f'"generate_release_notes":{generate_release_notes}'

    response = requests.post(url=f'https://api.github.com/repos/{owner}/{repo}/releases',
                             headers=headers,
                             data=data)
    print(response.text)


def main(token: str, owner: str, repo: str):
    env_file = os.getenv('GITHUB_ENV')

    release = get_github_latest_release(token=token, owner=owner, repo=repo)

    data = release.json()
    if data.get('message', '') == '':
        # message only exists if no releases exist
        # we have a release, get the current release number and
        # strip the v prefix
        current_release_tag = data.get('tag_name').replace('v', '')
        logger.info(f'Releases exist. Current release is {current_release_tag}')

        new_release_tag = f'v{semver.bump_patch(version=current_release_tag)}'
        logger.info(f'New release created: {new_release_tag}')
        create_github_release(
            token=token,
            owner=owner,
            repo=repo,
            tag=new_release_tag,
            generate_release_notes=True)
        with open(env_file, "a") as f:
            f.write(f"RELEASE_TAG={new_release_tag}")

    else:
        # message property exists and has a value
        # create a new release so we can start
        logger.info('No releases exist. Setting original tag to v0.0.1')
        current_release_tag = 'v0.0.1'
        create_github_release(
            token=token,
            owner=owner,
            repo=repo,
            tag=current_release_tag,
            generate_release_notes=False)
        with open(env_file, "a") as f:
            f.write(f"RELEASE_TAG={current_release_tag}")


if __name__ == '__main__':
    load_dotenv()
    cvss_rescore_gh_token = os.getenv('CVSS_RESCORE_GH_TOKEN')
    owner = os.getenv('OWNER')
    repo = os.getenv('REPO')

    main(token=cvss_rescore_gh_token, owner=owner, repo=repo)
